module.exports = (app, router) => {
    const Anime = app.db.models.ani.series;

    const editCheck = (series, req, res) => {
        if (!series) {res.status(400).send("A series with that ID doesn't exist!"); return 0;}
        if (series.meta.locked) {res.status(401).send("This series has been locked. Nobody can edit it unless it has been unlocked. Please contact API admin to have it unlocked if you believe there is erroneous information."); return 0;}
        if (series.meta.approved && !series.meta.locked && req.unauthorized) {res.status(401).send("This series has been approved, so you must have series approval permissions in order to do that."); return 0;}
        if (series.meta.creator !== req.authenticatedUser && req.unauthorized) {res.status(401).send("You must have series approval permissions or be the series' creator to do that !"); return 0;}
        if (!req.authenticatedUser || !req.authenticatedUser.id) {res.status(401).send("You are not authorized to do that!"); return 0;}
        return true;
    }

    const pushEdit = (action, req, series) => {
        series.meta.edits.push({
            user: req.authenticatedUser.id,
            timestamp: new Date().getTime(),
            action
        });
        series.markModified('meta.edits');
    };

    const edits = app.util.editString(pushEdit, editCheck, router, Anime);

    router.route('/:id/synopsis') //completed i think?
        .patch(app.auth.token, app.auth.perms('series-submit'), async (req, res, next) => {
            if (!req.params.id) {return;}
            const series = await Anime.findOne({id: req.params.id.toLowerCase()});
            if (!editCheck(series, req, res)) {return;}
            if (!req.body.synopsis) {return res.status(400).send("You did not provide a new synopsis in your body.");}
            if (req.body.synopsis.length > 1000) {return res.status(400).send("Your new synopsis is too long!");}
            try {
                series.synopsis.synopsis = req.body.synopsis;
                if (!series.synopsis.by) {series.synopsis.by = req.authenticatedUser.id;}
                if (req.body.updateAuthor === true && series.synopsis.by !== req.authenticatedUser.id) {series.synopsis.by = req.authenticatedUser.id;} // TODO doc this extraneous body item
                pushEdit("Updated synopsis", req, series);
                series.markModified('synopsis.synopsis');
                return series.save()
                    .then(() => res.send("Synopsis updated."))
                    .catch(() => res.status(500).send("There was an error trying to update your synopsis. Please try again."));
            }
            catch {return res.status(500).send("There was an error trying to update your synopsis. Please try again.");}
        })
        .get(app.auth.tokenPass, app.auth.permsPass('series-approve'), async (req, res) => { //working
            const series = await Anime.findOne({id: req.params.id.toLowerCase()});
            if (!series || (series && !series.meta.completed && req.unauthorized)) {return res.status(400).send("A series with that ID doesn't exist!");}
            return res.send({synopsis: series.synopsis.synopsis, by: series.synopsis.by});
        });

    [
        ['name', x => x.match(/^[\w_\-!?.:; ]+$/gm) && x.length < 150],
        ['romaji', x => x.length < 150],
        ['kanji', x => x.length < 150]

    ].forEach(field => edits.stringWrap(field[0], field[1]));

    /*router.patch('/:id/name', app.auth.token, app.auth.perms('series-submit'), async (req, res) => {
        if (!req.params.id) {return;}
        const series = await Anime.findOne({id: req.params.id.toLowerCase()});
        if (!editCheck(series, req, res)) {return;}
        if (!req.body.name) {return res.status(400).send("You did not provide a new name in your body.");}
        if (!req.body.name.match(/^[\w_\-!?.:; ]+$/gm) || req.body.name.length > 150) {return res.status(400).send("Your new name is too long or contains illegal characters.");}
        try {
            series.name = req.body.name.trim();
            pushEdit("Updated name", req, series);
            return series.save()
                .then(() => res.send("Name updated."))
                .catch(() => res.status(500).send("There was an error trying to update your name. Please try again."));
        } catch {return res.status(500).send("There was an error trying to update your name. Please try again.");}
    });

    router.patch('/:id/romaji', app.auth.token, app.auth.perms('series-submit'), async (req, res) => {
        if (!req.params.id) {return;}
        const series = await Anime.findOne({id: req.params.id.toLowerCase()});
        if (!editCheck(series, req, res)) {return;}
        if (!req.body.romaji) {return res.status(400).send("You did not provide a new romanized name (romaji) in your body.");}
        if (req.body.romaji.length > 150) {return res.status(400).send("Your new romanized name is too long or contains illegal characters.");}
        try {
            series.romaji = req.body.romaji.trim();
            pushEdit("Updated romaji", req, series);
            return series.save()
                .then(() => res.send("Romanized name updated."))
                .catch(() => res.status(500).send("There was an error trying to update your romaji. Please try again."));
        } catch {return res.status(500).send("There was an error trying to update your romaji. Please try again.");}
    });

    router.patch('/:id/kanji', app.auth.token, app.auth.perms('series-submit'), async (req, res) => {
        if (!req.params.id) {return;}
        const series = await Anime.findOne({id: req.params.id.toLowerCase()});
        if (!editCheck(series, req, res)) {return;}
        if (!req.body.kanji) {return res.status(400).send("You did not provide a new kanji in your body.");}
        if (req.body.kanji.length > 150) {return res.status(400).send("Your new kanji is too long or contains illegal characters.");}
        try {
            series.kanji = req.body.kanji.trim();
            pushEdit("Updated kanji", req, series);
            return series.save()
                .then(() => res.send("Kanji updated."))
                .catch(() => res.status(500).send("There was an error trying to update your kanji. Please try again."));
        } catch {return res.status(500).send("There was an error trying to update your kanji. Please try again.");}
    });*/



    //router.use('/:id/altnames', app.auth.tokenPass, app.auth.permsPass('series-approve'))

};